/****************************************************************************/
/* Ecmp 338, Assignment #6, Part III                                        */
/*      'consumer.c' - Contains all of the code necessary to make a         */
/*              rootin', tootin', consumin' client!                         */
/* by Devon Smith (devon@ces.cwru.edu) and                                  */
/*    Andy Reitz (reitz@ces.cwru.edu)                                       */
/* Date: 4/26/98                                                            */
/****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include "bm.h"

/*
 * This function should be executed when the consumer is killed!
 */
void sigterm_handler (int isignal)
{
	fprintf (stderr, "Got the kill signal, bye-bye.\n");
	exit (0);
} /* End sigterm_handler(). */

/*
 * This function is to use the buffer management functions in order to
 * consume (remove an item) from the buffer. At this point, it's pretty
 * much going to be a wrapper for ConsumerBufferElement, but eventually,
 * it'll be in it's own process, and include some code to randomly
 * consume from the buffer, and exit gracefully.
 */
int consume (char *host)
{
	int counter = 0;
	CLIENT *clnt;

	int *data_consumed;	/* The data to be eaten... */

	/* Variables for the _GAY_ RPC functions. */
	char *consumerbufferelement_2_arg;
	int *result_4;
	char *initializebuffer_2_arg;

	/* Attach to the server... ? */
	clnt = clnt_create(host, BM, ONE, "netpath");
	if (clnt == (CLIENT *) NULL)
		{
		clnt_pcreateerror(host);
		exit(1);
		}

	/* Initialize the buffer management process. */
	result_4 = initializebuffer_2((void *)&initializebuffer_2_arg, clnt);
	if (result_4 == (int *) NULL)
		{
		clnt_perror(clnt, "InitializeBuffer() call failed");
		}

	/* Seed, my friend, seed. */
	srand ((unsigned) getpid());

	/*
	 * This process isn't going to consume un-maligned, until it has gone
	 * too far, then we'll manually shut it down.
	 */
	while (counter < MAX_PRODUCE)
		{
		/* Start every loop off with some sort of random delay. */
		sleep (rand() % 10);

		/* Now, try and get some data from the buffer. */
		data_consumed = consumerbufferelement_2((void *)&consumerbufferelement_2_arg, clnt);
		if (data_consumed == (int *) NULL)
			{
			clnt_perror(clnt, "CBE call failed");
			clnt_destroy(clnt);
			return (FALSE);
			}

		/*
		 * If we get a big fat ERROR, then just say so, and keep
		 * trying -- we're assuming that the producer is still
		 * rockin'.
		 */
		if (*data_consumed == ERROR)
			{
			printf ("Tried to consume from empty buffer.\n");
			}
		else
			{
			/* Well, if we got here -- then it all went well, eh? */
			printf ("%d was consumed.\n", *data_consumed);
			}

		counter++;
		}		

	clnt_destroy(clnt);
	return (TRUE);
} /* End consume(). */

/*
 * The main for the consumer function, pretty much kicks off the consume()
 * function.
 */
int main(int argc, char *argv[])
{
	char *host;

	/* Grab the program-quitting signals. */
	signal (SIGTERM, sigterm_handler);
	signal (SIGQUIT, sigterm_handler);
	signal (SIGINT, sigterm_handler);

	/* Error-check the parameters. */
	if (argc < 2)
		{
		printf("usage: %s server_host\n", argv[0]);
		exit(1);
		}

	/* Launch! */
	host = argv[1];

	if (consume (host) == TRUE)
		/* Everything went okay, just shut it down. */
		kill (getpid(), SIGTERM);
	else
		{
		/* Hurm... Had some problems, eh? */
		fprintf (stderr, "Burp! Had some problems consuming...\n");
		kill (getpid(), SIGTERM);
		}

	/* Shouldn't ever be here... */
	return (2);
} /* End main(). */
