/****************************************************************************/
/* Ecmp 338, Assignment #6, Part III                                        */
/*      'producer.c' - Contains all of the code necessary in order to       */
/*              create a producer process.                                  */
/* by Devon Smith (devon@ces.cwru.edu) and                                  */
/*    Andy Reitz (reitz@ces.cwru.edu)                                       */
/* Date: 4/26/98                                                            */
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include "bm.h"

/*
 * This function should be executed when the producer is killed!
 */
void sigterm_handler (int isignal)
{
	fprintf (stderr, "Got the kill signal, bye-bye.\n");
	exit (0);
} /* End sigterm_handler(). */

/*
 * This function represents the producer process, and as such, it performs
 * the following:
 *      1. Generate a random number (our production!).
 *      2. Insert this number into the shared buffer:
 *         a) Determine the next free index.
 *         b) Call the buffer management function, to get it done.
 *      3. Assuming that nothing failed, write some output, and be done.
 *         a) If either of those functions fail, return an errorcode to
 *            the calling procedure.
 */
int produce (char *host)
{
	int num_prod;		/* The number produced. */
	int counter = 0;
	int *idx;

	int PBE_param[2];

	/* Variables for these really _GAY_ RPC functions. */
	CLIENT *clnt;
	char *elementindextoproduce_2_arg;
	int *result_2;
	int producebufferelement_2_arg;
	int *result_4;
	char *initializebuffer_2_arg;

	/* Attach to server? */
	clnt = clnt_create(host, BM, ONE, "netpath");
	if (clnt == (CLIENT *) NULL)
		{
		clnt_pcreateerror(host);
		exit(1);
		}

	/* Initialize the buffer management process. */
	result_4 = initializebuffer_2((void *)&initializebuffer_2_arg, clnt);
	if (result_4 == (int *) NULL)
		{
		clnt_perror(clnt, "InitializeBuffer() call failed");
		}

	/* Seed our randomness... */
	srand ((unsigned) getpid());

	/*
	 * Theoretically, this process should run forever. In reality,
	 * however, this is probably bad, so it will go until it's
	 * iterated MAX_PRODUCE times.
	 */
	while (counter < MAX_PRODUCE)
		{
		num_prod = rand() % 99 + 1;
		printf ("%d was generated by producer.\n", num_prod);

		idx = elementindextoproduce_2((void *)&elementindextoproduce_2_arg, clnt);
		if (idx == (int *) NULL)
			{
			clnt_perror(clnt, "EITP call failed");
			clnt_destroy(clnt);
			return (ERROR);
			}

		/*
		 * Check to see if there's free space. If there isn't -- we
		 * keep going. We assume that some consumption will occur,
		 * and we'll able to produce more later.
		 */
		if (*idx == ERROR)
			{
			/* Complain. */
			printf ("Producer: No free space in buffer.\n");
			}
		else
			{
			/*
			 * Got an index, let's insert it. Calling
			 * ProduceBufferElement() requires our parameters (the
			 * index, and the data), to be stuffed into an array,
			 * and a pointer to said array to be passed over the
			 * RPC pipe.
			 */
			PBE_param[0] = *idx;
			PBE_param[1] = num_prod;
			result_2 = producebufferelement_2(PBE_param, clnt);
			if (result_2 == (int *) NULL)
				{
				clnt_perror(clnt, "PBE call failed");
				clnt_destroy(clnt);
				return (ERROR);
				}

			if (*result_2 == ERROR)
				{
				printf ("Producer: Insert into buffer failed.\n");
				clnt_destroy(clnt);
				return (ERROR);
				}
			}

		/* Randomize our production-schedule. */
		sleep ((int) (num_prod / 10));
		counter++;
		}

	clnt_destroy(clnt);
	return (TRUE);
} /* End produce(). */

/*
 * The main() for this process pretty much just kicks off the produce()
 * function.
 */
int main(int argc, char *argv[])
{
	char *host;

	/* Grab the program-quitting signals. */
	signal (SIGTERM, sigterm_handler);
	signal (SIGQUIT, sigterm_handler);
	signal (SIGINT, sigterm_handler);

	/*
	 * Arrr! Error check the command line parameters, matey!
	 */
	if (argc < 2)
		{
		printf("Usage: %s server_host\n", argv[0]);
		exit(1);
		}

	host = argv[1];

	if (produce (host) == TRUE)
		/* Everything went okay, shut it down. */
		kill (getpid(), SIGTERM);
	else
		{
		/* Had an error. Bummer. */
		fprintf (stderr, "Producer terminating abnormally.\n");
		kill (getpid(), SIGTERM);
		}

	/* Shouldn't ever be here... */
	return (2);
} /* End main(). */
